# vim:spell:spelllang=fr
# -*- encoding: utf-8 -*-

import sys
from optparse import make_option
import xml.etree.ElementTree as etree

from django.core.management.base import BaseCommand

from polynum.base.models import Entity, EntityRelation
from polynum.utils import slice_collection

from polynum.ldif import LDIFParser

class LdapEntity(object):
    def __init__(self, **kwargs):
        self.__dict__.update(kwargs)
        self.depth = 0

    def __unicode__(self):
        return u'<LdapEntity: %s>' % (u', '.join(u'%s=%s' % (key, value) for key, value in self.__dict__.iteritems()))

class UpdXML(object):
    def __init__(self, input):
        self.input = input
        self.entities = {}

    def parse(self):
        element_tree = etree.parse(self.input)
        for structure in element_tree.findall('structure'):
            code = structure.get('code').lower()
            libelle_long = structure.find('libelle_long').text
            libelle_court = structure.find('libelle_court').text
            type_ = structure.find('type').text
            pere = structure.findall('pere')
            if pere is not None:
                code_pere = map(lambda p: p.text.lower(), pere)
                code_pere = filter(lambda x: x != 'sri', code_pere)
            self.entities[code] = LdapEntity(code=code,
                    name=libelle_court,
                    parents=code_pere,
                    description=libelle_long,
                    type=type_)
            if len(self.entities) % 100 == 0:
                print 'Parsed %06s\r' % len(self.entities),
                sys.stdout.flush()
        self.finish()

    def finish(self):
        for entity in self.entities.values():
            entity.parents = [self.entities[parent] for parent in entity.parents]
        print 'Parsed %06s\r' % len(self.entities)

class UpdLDIF(LDIFParser):

    def __init__(self, input):
        LDIFParser.__init__(self, input)
        self.entities = {}

    def handle(self, dn, entry):
        if not dn:
            return
        if 'supannCodeEntite=' in dn:
            parents = [parent.decode('utf-8') for parent in entry.get('supannCodeEntiteParent', [])]
            code = entry.get('supannCodeEntite')[0]
            if code in self.entities:
                raise "double entity", code
            name = ''.join(entry.get('ou') or [])
            description = ''.join(entry.get('description') or [])
            self.entities[code] = LdapEntity(code=code.decode('utf-8'),
                    name=name.decode('utf-8'),
                    parents=parents,
                    description=description.decode('utf-8'))
            if len(self.entities) % 100 == 0:
                print 'Parsed %06s\r' % len(self.entities),
                sys.stdout.flush()
            self.finish()

    def finish(self):
        for entity in self.entities.values():
            entity.parents = [self.entities[parent] for parent in entity.parents]
        print 'Parsed %06s\r' % len(self.entities)

class Command(BaseCommand):
    '''
       Recharge la liste des entités
    '''
    can_import_django_settings = True
    output_transaction = True
    requires_model_validation = True
    args = ''
    help = 'Recharge la liste des entités'

    option_list = BaseCommand.option_list + (
            make_option("--delete"),
            make_option("--xml", action="store_true"))

    def build_parents_relations(self, relations, entity, current=None, direct=True, depth=0):
        if not current:
            current = entity
        entity.depth = max(entity.depth, depth)
        for parent_entity in current.parents:
            relations.add((parent_entity, entity, direct))
            self.build_parents_relations(relations, entity, current=parent_entity, direct=False, depth=depth+1)

    def load_file(self, filepath, **options):
        if 'xml' in options:
            parser = UpdXML(file(filepath))
            parser.parse()
        else:
            parser = UpdLDIF(file(filepath))
            parser.parse()
        relations = set()
        for entity in parser.entities.values():
            self.build_parents_relations(relations, entity)
        EntityRelation.objects.filter().delete()
        for sl in slice_collection(list(Entity.objects.all()), 100):
            Entity.objects.filter(code__in=(o.code for o in sl)).delete()
        for entity in parser.entities.values():
            entity.django_instance = \
                Entity(code=entity.code, name=entity.name,
                        description=entity.description, depth=entity.depth)
        i = 0
        for sl in slice_collection([entity.django_instance for entity in parser.entities.values()], 10):
            Entity.objects.bulk_create(sl)
            i += len(sl)
            print 'Added %06s entities\r' % i,
            sys.stdout.flush()
        print
        i = 0
        entity_relations = list(EntityRelation(parent=parent.django_instance, child=child.django_instance, direct=direct) for parent, child, direct in relations)
        for sl in slice_collection(entity_relations, 50):
            EntityRelation.objects.bulk_create(sl)
            i += len(sl)
            print 'Added %06s entity relations\r' % i,
            sys.stdout.flush()
        print

    def handle(self, *args, **options):
        for filepath in args:
            self.load_file(filepath, **options)

