# -*- coding: utf-8 -*-
"""
PolyNum

Définition des entités administratives
"""

from django.db import models
from django.utils.translation import ugettext_lazy as _

class EntityType(models.Model):

    class Meta:
        app_label = 'base'
        verbose_name = _(u"Type d'entité administrative")
        verbose_name_plural = _(u"Type d'entités administratives")

    def __unicode__(self):
        return u'%s (%s)' % (self.description, self.name)

    name = models.CharField(max_length=128, verbose_name=_(u'Nom'))
    description = models.TextField(verbose_name=_(u'Description'), blank=True)


class Entity(models.Model):

    class Meta:
        app_label = 'base'
        verbose_name = _(u'Entité administrative')
        verbose_name_plural = _(u'Entités administratives')
        ordering = ('depth', 'name')

    def __unicode__(self):
        return self.get_description() or self.get_name()

    code = models.CharField(primary_key=True, max_length=64, verbose_name=_(u'Code'))
    is_active = models.BooleanField(verbose_name=_(u'Active'), default=True, blank=True)
    name = models.CharField(max_length=128, verbose_name=_(u'Nom'), blank=True)
    description = models.TextField(verbose_name=_(u'Description'), blank=True)
    types = models.ManyToManyField(EntityType, null=True, blank=True)
    name_override = models.CharField(max_length=128, verbose_name=_(u'Nom spécifique PolyNum'), blank=True)
    description_override = models.TextField(verbose_name=_(u'Description spécifique PolyNum'), blank=True)
    depth = models.PositiveIntegerField(default=0)

    def children(self):
        return Entity.objects.filter(parent_relations__parent=self)

    def parents(self):
        return Entity.objects.filter(children_relations__child=self)

    def accountings(self):
        '''
        return a set of "accounting" codes: one of the entity and those of all its parents
        '''
        ret = set()
        if self.accounting:
            ret.add(self.accounting)
        return sorted(ret.union([acc for acc in self.parents().values_list('accounting', flat=True) if acc]))

    def get_name(self):
        return self.name_override or self.name
    get_name.short_description = _(u'Nom')

    def get_description(self):
        return self.description_override or self.description
    get_description.short_description = _(u'Description')

class AccountingCode(models.Model):
    class Meta:
        verbose_name = _('Code de comptabilité')
        verbose_name_plural = _('Codes de comptabilité')
        ordering = ('entity', 'code')

    entity = models.ForeignKey(Entity)
    code = models.CharField(max_length=32, verbose_name=_('Code financier'))

class EntityRelation(models.Model):
    parent = models.ForeignKey(Entity, related_name='children_relations', verbose_name=_(u'Entité parente'), on_delete=models.DO_NOTHING)
    child  = models.ForeignKey(Entity, related_name='parent_relations', verbose_name=_(u'Entité fille'), on_delete=models.DO_NOTHING)
    direct = models.BooleanField(default=True, blank=True)

    class Meta:
        app_label = 'base'
