# -*- coding: utf-8 -*-

from django import forms
from django.utils.translation import ugettext_lazy as _
from django.core.exceptions import ValidationError

from crispy_forms.helper import FormHelper
from crispy_forms.layout import Layout, Div, HTML, Field, Fieldset

from ..base.models import (Request, Profile, ProfileOption,
        ProfileOptionChoice)
from . import utils

class BaseModelForm(forms.ModelForm):
    def __init__(self, *args, **kwargs):
        super(BaseModelForm, self).__init__(*args, **kwargs)
        for field in getattr(self.Meta, 'required', ()):
            self.fields[field].required = True
        self.helper = FormHelper()
        self.helper.form_tag = False

    def full_clean(self):
        oldvalues = {}
        for k, v in self.fields.iteritems():
            oldvalues[k] = v.required
            v.required = False
        result = super(BaseModelForm, self).full_clean()
        for k, v in oldvalues.iteritems():
            self.fields[k].required = v
        return result

    def pprint_data(self, data):
        ret = []
        required = getattr(self.Meta, 'required', ())
        for field in self._meta.fields:
            hook_name = 'pprint_data_%s' % field
            hook = getattr(self, hook_name, None)
            if hook is None:
                value = getattr(self.instance, field, '') or ''
                missing = not bool(value) and field in required
                ret.append((self.fields[field].label, u'%s' % (getattr(self.instance, field, None) or ''), missing))
            else:
                ret.append(hook(data))
        return ret

class DocumentUploadForm(BaseModelForm):
    class Meta:
        model = Request 
        fields = ('uploadfile', )
        widgets = {
                'uploadfile': forms.ClearableFileInput,
        }

    @classmethod
    def display_name(self):
        return _(u'Envoi du document')

    def __init__(self, *args, **kwargs):
        super(DocumentUploadForm, self).__init__(*args, **kwargs)
        self.fields['uploadfile'].label = _(u'Document, en format PDF ou papier')

        # crispy
        self.helper.layout = Layout(
                HTML(_(u"""
                <p>Téléchargez ici le document que vous désirez envoyer au service
                reprographie.</p>
                """)),
                Field('uploadfile'),
                )
    def clean(self):
        cleaned_data = super(DocumentUploadForm, self).clean()
        if cleaned_data.get('uploadfile'):
            try:
                utils.fill_document_attributes_from_pdf_file(self.instance, self.cleaned_data['uploadfile'])
            except ValueError, e:
                raise ValidationError(e.args[0])
        return cleaned_data

    def save(self, *args, **kwargs):
        return super(DocumentUploadForm, self).save(*args, **kwargs)

    def pprint_data_uploadfile(self, data):
        '''
        get the "cleaned" datas from the form, return a list of tuples
        [ (u'description', u'value'), ... ]
        '''
        return (_(u'Document'), self.instance.uploadfile
            or _(u'sous forme papier'), False)


class DocumentDetailsForm(BaseModelForm):
    class Meta:
        model = Request
        fields = ('name', 'nb_pages', 'usage', )
        required = fields

    @classmethod
    def display_name(self):
        return _(u'Détails sur le document')

    def __init__(self, *args, **kwargs):
        super(DocumentDetailsForm, self).__init__(*args, **kwargs)
        self.helper.layout = Layout(
                Field('name', css_class='span12'),
                Field('nb_pages', css_class='input-mini'),
                Field('usage', css_class='span6'),
                )

class ReproOriginForm(BaseModelForm):
    class Meta:
        model = Request
        fields = ('sponsor', 'entity')
        widgets = { 'entity': forms.TextInput }
        required = ('entity', )

    @classmethod
    def display_name(self):
        return _(u'Affectation de la demande')

    def __init__(self, *args, **kwargs):
        super(ReproOriginForm, self).__init__(*args, **kwargs)
        # crispy
        self.helper = FormHelper()
        self.helper.form_tag = False
        self.helper.layout = Layout(
                Field('sponsor', css_class='span12'),
                Field('entity', template = 'entityselector.html'))

class ReproDetailsForm(BaseModelForm):
    class Meta:
        model = Request
        fields = ('copies', 'base_profile', 'details',)
        widgets = {
            'base_profile': forms.RadioSelect,
            'details': forms.Textarea,
        }

    @classmethod
    def display_name(self):
        return _(u'Choix de reprographie')

    def __init__(self, *args, **kwargs):
        super(ReproDetailsForm, self).__init__(*args, **kwargs)
        l = ((u'%s' % p.id, p) for p in Profile.objects.all()) 
        self.fields['base_profile'].choices = l

        # crispy
        self.helper = FormHelper()
        self.helper.form_tag = False
        self.helper.layout = Layout(
                Field('copies', css_class='input-mini'),
                Field('base_profile', template = 'radioprofile.html'),
                )

        self.helper.layout.fields.extend([
            HTML(u'<div class="accordion-group">'),
            HTML(u'<div class="accordion-heading"><a class="accordion-toggle" data-toggle="collapse" href="#profile-details">%s</a></div>' % _(u'Options')),
            HTML(u'<div id="profile-details" class="accordion-body collapse">'),
            ])

        self.optionfields = []
        # build "options" fields
        for option in ProfileOption.objects.filter(visible=True).prefetch_related('profileoptionchoice_set'):
            fieldname = 'profile_option_%s' % option.id
            self.optionfields.append(fieldname)
            choices = [(u'%s' % c.id, c.name) \
                    for c in option.profileoptionchoice_set.all()]
            if option.list_type == 'L': # Liste (choix unique)
                self.fields[fieldname] = forms.ChoiceField(
                    label = u'%s' % option.name,
                    choices = choices,
                    required = False,
                    help_text = option.description,
                    )
            elif option.list_type == 'R': # Radio (choix unique)
                self.fields[fieldname] = forms.ChoiceField(
                    label = u'%s' % option.name,
                    widget = forms.RadioSelect,
                    choices = choices,
                    required = False,
                    help_text = option.description,
                    )
            elif option.list_type == 'C': # Cases à cocher (choix multiple)
                self.fields[fieldname] = forms.MultipleChoiceField(
                    label = u'%s' % option.name,
                    widget = forms.CheckboxSelectMultiple,
                    choices = choices,
                    required = False,
                    help_text = option.description,
                    )
            elif option.list_type == 'LM': # Liste (choix multiple)
                self.fields[fieldname] = forms.MultipleChoiceField(
                    label = u'%s' % option.name,
                    choices = choices,
                    required = False,
                    help_text = option.description,
                    )

        # hack to display fields side-by-side (should be done in a template)
        for n in xrange(len(self.optionfields)/2):
            left_field = Field(self.optionfields[n*2])
            try:
                right_field = Field(self.optionfields[n*2+1])
            except:
                right_field = HTML('')
            self.helper.layout.fields.append(Div(
                Div(left_field, css_class='span6'),
                Div(right_field, css_class='span6'),
                css_class="row",
                ))

        self.helper.layout.fields.extend([
            Field('details', css_class='span12'),
            HTML(u'</div></div>'), # profile_details
            ])

    def save(self, *args, **kwargs):
        self.instance.choices.clear()
        chosen = set()
        for option in ProfileOption.objects.filter(visible=True):
            fieldname = 'profile_option_%s' % option.id
            cleaned_data = self.cleaned_data
            value = cleaned_data.get(fieldname, None)
            if value is not None:
                chosen.add(value)
        chosen = filter(None, chosen)
        result = super(ReproDetailsForm, self).save(*args, **kwargs)
        self.instance.choices = ProfileOptionChoice.objects.filter(id__in=chosen)
        return result


    def pprint_data_base_profile(self, data):
        profile = getattr(self.instance, 'base_profile', None)
        if not profile:
            base_profile = data.get('base_profile')
            if base_profile:
                profile = Profile.objects.get(id=base_profile)
        value = u'%s' % profile if profile else ''
        return (_(u'Profil de base'), value, not bool(value))

    def pprint_data_choices(self, data):
        if self.instance.id:
            for choice in self.instance.choices.all():
                ret.append((choice.option.name, choice.name))
        else:
            option_choices = filter(None, [ self.data.get(field) for field in self.optionfields ])
            for choice in ProfileOptionChoice.objects.filter(id__in=option_choices).select_related():
                ret.append((choice.option.name, choice.name, False))
        if self.instance.details:
            ret.append((_(u'Prestations complétementaires'), _(u'oui'), False))
        return ret


class DeliveryForm(BaseModelForm):
    class Meta:
        model = Request
        fields = ('delivery_place', 'delivery_date', 'contact_email',
                'contact_telephone1', 'contact_telephone2', 'contact_bureau')
        required = ('delivery_place', 'delivery_date')

    @classmethod
    def display_name(self):
        return _(u"Options de livraison")

    def __init__(self, *args, **kwargs):
        super(DeliveryForm, self).__init__(*args, **kwargs)
        self.fields['delivery_place'].empty_label = None

        # crispy
        self.helper = FormHelper()
        self.helper.form_tag = False
        self.helper.layout = Layout(
            Field('delivery_place', css_class='span6'),
            Field('delivery_date', css_class='span2 datepicker'),
            Fieldset(u'Contact',
                Field(
                    'contact_email',
                    'contact_telephone1',
                    'contact_telephone2',
                    'contact_bureau')))


class CopyrigtsForm(BaseModelForm):
    class Meta:
        model = Request
        fields = ('copyright', 'licence')
        required = fields
        widgets = {
        }

    @classmethod
    def display_name(self):
        return _(u"Droits d'auteur")

    def __init__(self, *args, **kwargs):
        super(CopyrigtsForm, self).__init__(*args, **kwargs)
        self.fields['licence'].label = _(u'Diffusion de votre fichier')
        self.fields['copyright'].label = _(u'Droits de copie')
        if len(self.fields['licence'].choices) < 2:
            f = self.fields['licence'] 
            f.empty_label = None
            f.initial = list(self.fields['licence'].choices)[0][0]

        # crispy
        self.helper = FormHelper()
        self.helper.form_tag = False

    def pprint_data_copyright(self, data):
        missing = self.instance.copyright is None
        value = ''
        if self.instance.copyright is not None:
            value = _(u'Oui') if self.instance.copyright else _('Non')
        return (self.fields['copyright'].label, value, missing)

class FinancialForm(BaseModelForm):
    class Meta:
        model = Request
        fields = ('financial_code', 'financial_comment')
        required = ('financial_code',)

    @classmethod
    def display_name(self):
        return _(u"Données financières")

class CostForm(BaseModelForm):
    class Meta:
        model = Request
        fields = ('cost',)
        required = fields

    @classmethod
    def display_name(self):
        return _(u"Coût réel")

class ValidationForm(forms.Form):

    @classmethod
    def display_name(self):
        return _(u"Validation")

    def __init__(self, *args, **kwargs):
        super(ValidationForm, self).__init__(*args, **kwargs)
        # crispy
        self.helper = FormHelper()
        self.helper.form_tag = False

    def pprint_data(self, data):
        pass # no data in this step

