# -*- encoding: utf-8 -*-

from functools import wraps
import datetime
import time

import django.core.cache
from django.core.urlresolvers import RegexURLPattern, RegexURLResolver

def get_next_workable_day(today=None, hours=0, days=2, increment_hours=0, increment_days=1):
    if today is None:
        today = datetime.date.today()
    next = today + datetime.timedelta(hours=hours, days=days)
    while next.weekday() in (5,6):
        next = next + datetime.timedelta(hours=increment_hours, days=increment_days)
    return next

def profile(func):
    def f(*args, **kwargs):
        t = time.time()
        result = func(*args, **kwargs)
        print 'Profiling call to %s(%s, %s):' % (func, args, kwargs), time.time()-t
        return result
    return f

def tuplify(iterable):
    '''Convert all iterable to tuple.

       Use it when you want to index a dictionary with an iterable.
    '''
    if hasattr(iterable, '__iter__'):
        return tuple(tuplify(x) for x in iterable)
    else:
        return iterable

def stringify(iterable):
    if hasattr(iterable, '__iter__') and not isinstance(iterable, basestring):
        return unicode(map(stringify, iterable))
    else:
        return unicode(iterable)

def normalize_key(key):
    return ''.join(filter(lambda c: c.isalnum(), stringify(tuplify(key))))

class CachingDecorator(object):
    '''Mixin class for building caching decorators'''
    def __init__(self, **kwargs):
        self.__dict__.update(kwargs)

    def cache(self, args, kwargs):
        return None

    def key(self, args, kwargs):
        return tuple(args) + tuple(sorted(kwargs.iteritems()))

    def get(self, cache, key, default=None):
        return default

    def set(self, cache, key, value):
        pass

    def __call__(self, function):
        function_name = '{0}.{1}_cache'.format(function.__module__,
                function.__name__)
        @wraps(function)
        def f(*args, **kwargs):
            cache = self.cache(args, kwargs)
            key = normalize_key((function_name, self.key(args, kwargs)))
            result = self.get(cache, key)
            if result:
                return result
            result = function(*args, **kwargs)
            self.set(cache, key, result)
            return result
        return f

class CacheToSession(CachingDecorator):
    def cache(self, args, kwargs):
        return getattr(kwargs.get('request', None), 'session', None)

    def get(self, cache, key, default=None):
        if cache is not None:
            return cache.get(key, default)
        return default

    def set(self, cache, key, value):
        if cache is not None:
            cache[key] = value

'''Decorator adding caching to the request.session to any function'''
cache_to_session = CacheToSession()

class CacheToDjangoCache(CachingDecorator):
    '''Add cache using the Django cache framework to a function

       Parameters:
       cache_name - use a specific named cache backend
       timeout - cache values this amount of time
    '''
    def __init__(self, cache_name=None, timeout=None):
        self.cache_name = cache_name
        self.timeout = timeout

    def cache(self, args, kwargs):
        if self.cache_name:
            return django.core.cache.get_cache(self.cache_name)
        else:
            return django.core.cache.cache

    def get(self, cache, key, default=None):
        return cache.get(key, default)

    def set(self, cache, key, value):
        cache.set(key, value, timeout=self.timeout)

'''Decorator adding caching to the default Django cache backend'''
cache_to_django = CacheToDjangoCache

def slice_collection(iterable, length):
    '''Cut a slicable container into sample of maximum size equals to length'''
    if hasattr(iterable, '__getslice__'):
        i = 0
        while True:
            l = iterable[i:i+length]
            yield l
            if len(l) < length:
                break
            i += length
    else:
        l = []
        for x in iterable:
            l.append(x)
            if len(l) == length:
                yield l
                l = []
        if l:
            yield l

class DecoratedURLPattern(RegexURLPattern):
    def resolve(self, *args, **kwargs):
        result = super(DecoratedURLPattern, self).resolve(*args, **kwargs)
        if result:
            result.func = self._decorate_with(result.func)
        return result

class DecoratedRegexURLResolver(RegexURLResolver):
    def resolve(self, *args, **kwargs):
        result = super(DecoratedRegexURLResolver, self).resolve(*args, **kwargs)
        if result:
            result.func = self._decorate_with(result.func)
        return result

def decorated_includes(func, includes, *args, **kwargs):
    urlconf_module, app_name, namespace = includes

    for item in urlconf_module:
        if isinstance(item, RegexURLPattern):
            item.__class__ = DecoratedURLPattern
            item._decorate_with = func

        elif isinstance(item, RegexURLResolver):
            item.__class__ = DecoratedRegexURLResolver
            item._decorate_with = func

    return urlconf_module, app_name, namespace
